<?php
/*
  Plugin Name: Page Transitions 1.1
  Plugin URI: http://www.PageTransitions.com
  Description: The plugin adds amazing Page Transitions to your WordPress blog
  Author: Max Puhalevich
  Version: 1.1
  Author URI: http://wstune.com
 */

//Include config file
include_once('_config.php');

//Include all the modules
include_once(WPT_PLUGIN_PATH . '/modules/screenshot.php');
include_once(WPT_PLUGIN_PATH . '/modules/api.php');

//Add menu to the admin panel
add_action('init', 'wpt_init');
add_action('admin_menu', 'wpt_menu');
add_action('admin_head', 'wpt_head');
add_action('wp_head', 'wpt_head_frontend');
add_action('admin_notices', 'wpt_admin_notice');
register_activation_hook(__FILE__, 'wpt_activate');

//Get current version of the plugin
function wpt_version() {
	return '1.0';
}

//Init
function wpt_init() {
	wp_enqueue_script('swfobject');
	add_filter('home_template', 'wpt_home_template');
}

//Once plugin is activated
function wpt_activate() {

	//Create assets directory
	wp_mkdir_p(WP_CONTENT_DIR . '/ah_assets');

	//Concat all the page transition names into one line
	global $wpt_transitions, $wpt_default_options;
	$used_transitions = '';
	for ($i = 0; $i < count($wpt_transitions); $i++) {
		if ($i) {
			$used_transitions.= ',';
		}
		$used_transitions.= $wpt_transitions[$i][0];
	}

	//Set default option values
	$options = $wpt_default_options;
	$options['wpt_used_transitions'] = $used_transitions;

	foreach ($options as $name => $value) {
		wpt_create_option($name, $value);
	}

	//Generate config file
	if (!$f = @fopen(WPT_PLUGIN_PATH . '/ah_files/config.xml', 'w')) {
		die('Set proper permissions for the plugin directory');
	}
	fputs($f, '<config><paths><transitions>' . WPT_PLUGIN_URL . '/ah_files/transitions</transitions><assets>' . WP_CONTENT_URL . '/ah_assets</assets></paths></config>');
	fclose($f);

	//Generate random password
	update_option('wpt_pass', wpt_generate_password());
	
	//Copy theme files
	wpt_move_theme_files();
}

function wpt_move_theme_files(){
	
	$theme_dir = STYLESHEETPATH . DIRECTORY_SEPARATOR;
	$plugin_dir = WPT_PLUGIN_PATH . DIRECTORY_SEPARATOR . 'theme-files' .DIRECTORY_SEPARATOR;
	
	//Website wrapper
	$file = $theme_dir . 'wpt-wrapper.php';
	if (!file_exists($file)){
		if(!@copy($plugin_dir . 'wpt-wrapper.php', $file)){
			die('Page Transitions Plugin Error: theme dir is not writable');
		}
	}
	
	//General js
	$file = $theme_dir . 'wpt-generic.js';
	if (!file_exists($file)){
		if(!@copy($plugin_dir . 'wpt-generic.js', $file)){
			die('Page Transitions Plugin Error: theme dir is not writable');
		}
	}
}

function wpt_home_template($template) {
	
	if($_GET['wpt-wrapped'] || get_option('wpt_off')){		
		return $template;
	}
	wpt_move_theme_files();
	
	return STYLESHEETPATH . DIRECTORY_SEPARATOR . 'wpt-wrapper.php';
}

function wpt_admin_notice() {
	if ($_GET['page'] == 'wpt_queue_page') {
		return;
	} else if (get_option('wpt_queue')) {
		echo '<div class="updated"><p>WPT plugin requires <a href="' . get_bloginfo('url') . '/wp-admin/admin.php?page=wpt_queue_page">user interaction</a>.</p></div>';
	}
}

//Sidebar navigation
function wpt_menu() {

	//Top page
	add_menu_page('Page Transitions', 'Page Transitions', 'level_10', 'wpt_top_page', 'wpt_settings_page');

	//Settings page
	add_submenu_page('wpt_top_page', 'Settings', 'Settings', 'level_10', 'wpt_settings_page', 'wpt_settings_page');

	//Notifications
	if (get_option('wpt_queue')) {
		$title = 'Notifications(!)';
	} else {
		$title = 'Notifications';
	}
	add_submenu_page('wpt_top_page', 'Notifications', $title, 'level_10', 'wpt_queue_page', 'wpt_queue_page');

	//Software page
	add_submenu_page('wpt_top_page', 'Software', 'Software', 'level_10', 'wpt_software_page', 'wpt_software_page');
	
	if (function_exists('remove_submenu_page')) {
		remove_submenu_page('wpt_top_page', 'wpt_top_page');
	} else {
		unset($GLOBALS['submenu']['wpt_top_page'][0]);
	}
}

//Transitions page
function wpt_settings_page() {

	//Save wished page transitions
	if ($_POST['save_transitions']) {
		$str = join(',', $_POST['wpt_used_transitions']);
		update_option('wpt_used_transitions', $str);
		$success = true;

		//Turn off page transitions
	} else if ($_POST['wpt_transitions_off']) {
		update_option('wpt_off', '1');
		$success = true;

		//Turn on page transitions
	} else if ($_POST['wpt_transitions_on']) {
		update_option('wpt_off', '');
		$success = true;

		//Add all the pages to the queue
	} else if ($_POST['reset_transitions']) {
		wpt_reset_transitions();
		$reset = true;

		//Save settings
	} else if ($_POST['save_settings']) {
		update_option('wpt_page_w', max(0, intval($_POST['wpt_page_w'])));
		update_option('wpt_page_h', max(0, intval($_POST['wpt_page_h'])));
		update_option('wpt_quality', max(0, intval($_POST['wpt_quality'])));
		$success = true;

		//Generate new password
	} else if ($_POST['new_pass']) {
		update_option('wpt_pass', wpt_generate_password());
		$success = true;
	}

	//Include the template
	include(WPT_PLUGIN_PATH . '/templates/settings.php');
}

//Queue page
function wpt_queue_page() {
	if ($_POST['wpt_ignore']) {
		update_option('wpt_queue', '');
	}
	include(WPT_PLUGIN_PATH . '/templates/queue.php');
}

//Software page
function wpt_software_page(){
	include(WPT_PLUGIN_PATH . '/templates/software.php');
}

//Prepare the string for usage in javascript code
function wpt_script_escape($str) {
	$str = trim($str);
	return str_replace(array("\n", "\r", "\t"), array('', '', ''), $str);
}

//Get all page transitions which are used
function wpt_get_used_transitions() {
	return explode(',', get_option('wpt_used_transitions'));
}

//Check if page transition is used on the website
function wpt_is_transition_on($transition) {
	global $wpt_used_transitions;
	if (!$wpt_used_transitions) {
		$wpt_used_transitions = wpt_get_used_transitions();
	}
	return in_array($transition, $wpt_used_transitions);
}

//Creates option if it is not exists
function wpt_create_option($name, $value) {
	if (!get_option($name)) {
		add_option($name, $value);
	}
}

//Add some javascript to the head
function wpt_head() {
	?>
	<script>
		var wpt_plugin_url = "<?php echo WPT_PLUGIN_URL ?>";
	</script>
	<?php
}

//Add some javascript to the head
function wpt_head_frontend() {
	?>
	<script src="<?php bloginfo('template_directory')?>/wpt-generic.js"></script>
	<?php
}

function wpt_generate_password($length = 8) {

	// start with a blank password
	$password = "";

	// define possible characters - any character in this string can be
	// picked for use in the password, so if you want to put vowels back in
	// or add special characters such as exclamation marks, this is where
	// you should do it
	$possible = "12346789bcdfghjkmnpqrtvwxyzBCDFGHJKLMNPQRTVWXYZ";

	// we refer to the length of $possible a few times, so let's grab it now
	$maxlength = strlen($possible);

	// check for length overflow and truncate if necessary
	if ($length > $maxlength) {
		$length = $maxlength;
	}

	// set up a counter for how many characters are in the password so far
	$i = 0;

	// add random characters to $password until $length is reached
	while ($i < $length) {

		// pick a random character from the possible ones
		$char = substr($possible, mt_rand(0, $maxlength - 1), 1);

		// have we already used this character in $password?
		if (!strstr($password, $char)) {
			// no, so it's OK to add it onto the end of whatever we've already got...
			$password .= $char;
			// ... and increase the counter by one
			$i++;
		}
	}

	// done!
	return $password;
}
?>