<?php

add_action('save_post', 'wpt_save_postdata');
add_action('added_post_meta', 'wpt_meta_changed');
add_action('updated_post_meta', 'wpt_meta_changed');
add_action('deleted_post_meta', 'wpt_meta_changed');

//Once post meta added / updated / deleted
function wpt_meta_changed($meta_id, $post_id = null) {
	
    if(!wpt_is_queue_post($post_id)){
  		return $post_id;
  	}
  	
  	//Invoke event
  	$post_id = apply_filters('wpt_add_post_to_queue', $post_id);
  	wpt_add_to_queue(get_permalink($post_id));
}

//Save post event listener
function wpt_save_postdata($post_id){
		
	//No actions for autosave
  	if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE){
  		return $post_id;
  	}
    
  	if(!wpt_is_queue_post($post_id)){
  		return $post_id;
  	}
  	
  	//Invoke event
  	add_post_to_queue($post_id);
}

//Check if we need to generate screenshot for the post
function wpt_is_queue_post($post_id){
	
	//Validate id
	if(!$post_id = intval($post_id)){
		return false;
	}
	
	//Get post  	
  	$post = get_post($post_id);
	
	//Check post type
  	global $wpt_post_types;
  	if(!in_array($post->post_type, $wpt_post_types)){
  		return false;
  	}
  	
  	//Check permissions
  	if($post->post_type == 'page' && !current_user_can('edit_page', $post_id)){
		return false;
	}else if(!current_user_can( 'edit_post', $post_id )){
		return false;
  	}
  	
  	//Check the status
  	if($post->post_status != 'publish'){
  		return false;
  	}
  	
  	return true;
}

//Add post to the queue list
function add_post_to_queue($post_id){
	if(!$post_id = intval($post_id)){
		return false;
	}
	if($post_id = apply_filters('wpt_add_post_to_queue', $post_id)){
  		wpt_add_to_queue(get_permalink($post_id));
  		do_action('wpt_post_queued', $post_id);
  	}
}

//Add url to the queue list
function wpt_add_to_queue($url){
	
	//Check if url is valid
	if(!wpt_is_valid_url($url)){
		return false;
	}
	
	//Check if such url is already in queue
	$urls = get_option('wpt_queue');
	if(in_array($url, $urls)){
		return false;
	}
	
	//Add url
	$urls[] = $url;
	update_option('wpt_queue', $urls);
	return true;
}

//Check if the url is valid and belongs to this domain
function wpt_is_valid_url($url){
	
	//Get WP domain
	$url_info = parse_url(get_bloginfo('url'));
	$domain = str_replace('.', '\.', $url_info['host']);
	
	//Validate url
	return preg_match('|^http://([a-z0-9-]+(\.[a-z0-9-]+)*\.)?'.$domain.'(:[0-9]+)?(/.*)?$|i', $url);
}

//Update all screenshots
function wpt_reset_transitions(){
	global $wpdb, $wpt_post_types;
	$sql = 'SELECT ID FROM '.$wpdb->posts.' 
			WHERE 
				post_type IN (';
	for($i = 0; $i < count($wpt_post_types); $i++){
		if($i){
			$sql.= ',';
		}
		$sql.='"'.$wpdb->escape($wpt_post_types[$i]).'"';
	}	
	$sql.= '	) 
				AND post_status = "publish"';
	if(!$items = $wpdb->get_results($sql)){
		return;
	}
	$urls = array();
	for($i = 0; $i < count($items); $i++){
		add_post_to_queue($items[$i]->ID);
	}
	
	//Add home page to the queue
	wpt_add_to_queue(get_bloginfo('url').'/?wpt-wrapped=1');
	
	//Invoke one more event
	do_action('wpt_reset_transitions');
	
	return true;
}

////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////

//AJAX: Update one page screenshot
function wpt_update_page_screenshot(){
	if($url = trim($_POST['url'])){
		$url = urldecode($url);
		
		//Check if is valid
		if(wpt_is_valid_url($url)){
			wpt_update_screenshots($url);
		}
	}
	
	exit();
}



?>